#############################################
## Lab # 01  STA483/583
##
## Goals:
##   - Basic intro to R
##   - Time series objects in R: class() = "ts"
##   - Basics of plotting a time series
##   - Smoothing and decomposition
##   
#############################################

########################################
## Example #1 - Reading in data
########################################
## Sheep Population in England and Wales by year
## in Thousands from 1867 to 1939

sheep <- scan("sheep.dat")
sheep <- ts(sheep, start=1867)
plot(sheep, main="Sheep Population in England and Wales", ylab="Population (thousands)",
     type='o', pch=20)
sheep.loess75 <- loess(sheep~time(sheep))  ## span=0.75 by default
sheep.loess50 <- loess(sheep~time(sheep), span=0.50)
sheep.loess25 <- loess(sheep~time(sheep), span=0.25)
lines(as.vector(time(sheep)), fitted(sheep.loess75), lwd=3, col="blue")
lines(as.vector(time(sheep)), fitted(sheep.loess50), lwd=3, col="darkgreen")
lines(as.vector(time(sheep)), fitted(sheep.loess25), lwd=3, col="purple2")

#############################################
## Example #2 - Data is also built into R
#############################################
## Monthly totals of airline passengers from 1949 to 1960
data(AirPassengers)  ## Load the data
AP <- AirPassengers  ## rename the data
help(AirPassengers)
plot(AP, lwd=2, col="blue", main="Monthly totals of International Airline Passengers",
     ylab="Thousands")
plot(AP, lwd=2, col="blue", main="Monthly totals of International Airline Passengers",
     ylab="Thousands", type='o')
plot(AP, lwd=2, col="blue", main="Monthly totals of International Airline Passengers",
     ylab="Thousands", type='o', pch=20)
##################################
## Best way to plot a time series???
## No "best" way, it largely depends on the amount of data you have, in my opinion
##  Lots of data -- a line suffices
##  Smaller amount of data -- line & dots, type='o', tends to look better (to me)
##  
## With time series data we typically "connect the dots" to show distinghuish it from 
## independent observations, a time series is a sequence.
class(AP)
frequency(AP)
start(AP)
end(AP)

## Output of decompose() function - uses Moving Average for trend
decompose(AP)
plot(decompose(AP))   ## Plot it

## Output of stl() function - uses LOESS for trend, we must specify an s.window
## generally s.window="periodic" works well, this takes seasonal means
help(stl)
stl(AP, s.window="periodic")   
plot(stl(AP, s.window="periodic"))
plot(stl(AP, s.window=7))    # This does LOESS on the seasonal terms

#############
## Let's explore the stl() function a bit more
ap.stl <- stl(AP, s.window="periodic")
class(ap.stl)
str(ap.stl)
head(ap.stl$time.series)
AP.trend <- ap.stl$time.series[,2]
plot(AP)
lines(AP.trend, col="blue", lwd=2)

####################
## We note the data appears to be heteroskedastic (variance changes)
## In particular: the variance is increasing with the mean
## That is, as the trend goes up, we see larger & larger jumps
log.AP <- log(AP)
plot(log.AP)   ## Variance seems stable now
plot(stl(log.AP, s.window="periodic"))

