#####################################
#####################################
## 
## This lab is all about predicting/foreasting
##
## So far in the class, we have primarily concentrated
## on modeling the time series.
## 
## In many cases, we can use those models to predict
## the future
##
## However, there is a mentality when it comes to forecasting
## that is doesn't matter how well the model fits, as long as
## it predicts well, that's good enough!!!
##
## Here we will look at the immigration data from the midterm
## We will explore everything on a log10 scale
## After spring break we will discuss how to go back to the original
## Series,
## 
## In this lab we explore all the tools in our arsenal for building
## a forecasting model.

####################
## First read in and setup data.
library(forecast)
immigrant <- read.csv("immigrants.csv")
immigrant <- ts(immigrant[,1], start=1933)
### We break the data into two groups,
## a 'training' set from which we will build our models
## and a 'testing' set for which we will compare our 
## predictive models
immigrant.train <- window(immigrant, end=2005)
immigrant.test <- window(immigrant, start=2006)
plot(immigrant.train, main="Late Life Immigrants", sub="Training Set")
log.immigrant.train <- log10(immigrant.train)
plot(log.immigrant.train, main="Late Life Immigrants", sub="Training Set", ylab="Log10(immigrants)")

####################################
## Forecasting!!!
####################################
##
## Some candidate Models from Class
## 
## Exponential Smoothing
## Holt Winter (no season), so exp smoothing with trend
## Linear Regression
## ARIMA approaches (i.e. differencing and ARMA)

#####################################
## First Exponential Smoothing &
## Holt Winters
exp.model <- HoltWinters(log.immigrant.train, beta=FALSE, gamma=FALSE)
exp.model
hw.model <- HoltWinters(log.immigrant.train, gamma=FALSE)
hw.model
##############################
## Exp Smoothing Model
## X[t+k] = 0.9999591*X[t] + (1-0.9999591)*X[t-1]
## 
## Note the coefficient, basically a Unit Root approach
## Very close to a random Walk approach
##
## Holt Winters
##
## X[t+1] = X_[t] + 0.0222*b[t]
## with
##    b[t] = 0.073664*(X_t[t] - X_[t-1] ) + (1-0.073664)*b[t-1]
##
## Very similiar to a Random Walk with drift
## We have the clear RW part, the X[t] plus a slight change term


##########################
## By deterministic model; i.e. regression
TIME <- 1:length(log.immigrant.train)
lm.fit <- lm(log.immigrant.train ~ TIME)
summary(lm.fit)
acf(lm.fit$residuals)
pacf(lm.fit$residuals)

### This sure looks like an AR(2)

Arima(lm.fit$residuals, order=c(1,0,0), include.mean=FALSE)
Arima(lm.fit$residuals, order=c(2,0,0), include.mean=FALSE)
Arima(lm.fit$residuals, order=c(3,0,0), include.mean=FALSE)
Arima(lm.fit$residuals, order=c(1,0,1), include.mean=FALSE)
Arima(lm.fit$residuals, order=c(0,0,2), include.mean=FALSE)

## all the AIC, AICc and BIC are picking the AR(2)
## also both terms seem significant.
ar2 <- Arima(lm.fit$residuals, order=c(2,0,0), include.mean=FALSE)
acf(ar2$residuals)
pacf(ar2$residuals)

### Refit the whole thing!
reg.model <- Arima(log.immigrant.train, order=c(2,0,0), xreg=TIME)
reg.model
###################
## My final regression model
## X[t] = immigrants, on log scale, at time t
##
## X[t] = 0.8311 + 0.0239*t + Z[t]
## where
## Z[t] = 0.9973*Z[t-1] - 0.3184*Z[t-2] + e[t] with e[t] ~ WN(0, 0.009912)


#################################
## By differencing, or ARIMA type approach
##
diff.log.immigrant <- diff(log.immigrant.train)
plot(diff.log.immigrant)
stats::acf(diff.log.immigrant)
stats::pacf(diff.log.immigrant)
mean(diff.log.immigrant)
var(diff.log.immigrant)
Arima(diff.log.immigrant)
qqnorm(diff.log.immigrant)
qqline(diff.log.immigrant)
#### Now refit the whole model
arima.model <- Arima(log.immigrant.train, include.drift=TRUE, order=c(0,1,0))
arima.model
############
## the differenced series looks like white noise.
## So we have a random walk with drift.
## X[t] = number of late life immigrants, on a log10 scale, at time t
##
##  X[t] = 0.0267 + X[t-1] + e[t]  where  e[t] ~ WN(0, 0.01251)

###################################
## We note, the drift term was barely significant (just outside 2 std.dev)
## Lets not model the drift term, see how things change, so just a random walk.
rw.model <- Arima(log.immigrant.train, include.drift=FALSE, order=c(0,1,0))
rw.model
###
## X[t] = X[t-1] + e[t] with e[t] ~ WN(0, 0.01322)

#####################################
## So we have 5 candidate models
##
##  Exponential Smoothing
##  Holt Winters
##  Regression + AR(2)
##  Random Walk with Drift
##  Random Walk


#######################################
## Let's compare the models in their ability
## to forecasting the testing set.
## First forecast the 'future'
exp.forecast <- forecast(exp.model, h=5)
hw.forecast <- forecast(hw.model, h=5)
reg.forecast <- forecast(reg.model, xreg=74:78)
arima.forecast <- forecast(arima.model, h=5)
rw.forecast <- forecast(rw.model, h=5)

#######################################
## A visual comparison
plot(exp.forecast, include=5, main="Exponential Smoothing Forecast", ylab="Log10(immigrants)")
lines(log10(immigrant.test), lwd=3, lty=3)
plot(hw.forecast, include=5, main="Holt Winters Forecast", ylab="Log10(immigrants)")
lines(log10(immigrant.test), lwd=3, lty=3)
plot(reg.forecast, include=5, main="Regression Forecast", ylab="Log10(immigrants)")
lines(log10(immigrant.test), lwd=3, lty=3)
plot(arima.forecast, include=5, main="Random Walk w/ Drift Forecast", ylab="Log10(immigrants)")
lines(log10(immigrant.test), lwd=3, lty=3)
plot(rw.forecast, include=5, main="Random Walk Forecast", ylab="Log10(immigrants)")
lines(log10(immigrant.test), lwd=3, lty=3)

################################################
## Let's compare the point predictions
exp.immigrant.predict <- exp.forecast$mean
hw.immigrant.predict <- hw.forecast$mean
reg.immigrant.predict <- reg.forecast$mean
arima.immigrant.predict <- arima.forecast$mean
rw.immigrant.predict <- rw.forecast$mean

tab <- cbind(log10(immigrant.test), 
             exp.immigrant.predict,
             hw.immigrant.predict,
             reg.immigrant.predict,
             arima.immigrant.predict, 
             rw.immigrant.predict)
colnames(tab) <- c("True Value", "Exp Smoothing",
                   "Holt Winters", "Regression",
                   "RW with Drift", "Random Walk")
tab

####################################################
## On average, which is the closest in terms of 
## mean squared prediction error

mspe <- rbind(mean( (log10(immigrant.test)-exp.immigrant.predict)^2 ),
              mean( (log10(immigrant.test)-hw.immigrant.predict)^2 ),
              mean( (log10(immigrant.test)-reg.immigrant.predict)^2),
              mean( (log10(immigrant.test)-arima.immigrant.predict)^2 ),
              mean( (log10(immigrant.test)-rw.immigrant.predict)^2 ) )
colnames(mspe) <- c("Mean Squared Prediction Error")
rownames(mspe) <- c("Exp Smoothing",
                   "Holt Winters", "Regression",
                   "RW with Drift", "Random Walk")
mspe

#########################
## It appears the regular Random Walk is the best at predicting
## our hold-out testing sample
##
## Exponential smoothing is just about the same, very close second
##
## Holt Winters & Random walk with drift are in second
## but an entire order of magnitude worse
##
## Regression was the worst performing with an additional
## order of magnitude worse
