#################################
## Simulated Cointegration Example.
##
library(tseries)
library(forecast)

############
## Generate the underlying "connection" series
set.seed(12345)
mu <- rep(2, 100)
for(i in 2:100) mu[i] <- mu[i-1] + rnorm(1)
mu <- ts(mu)
plot(mu)

x <- ts(0.5*mu + rnorm(100) )
y <- ts(0.75*mu + rnorm(100) )

plot(ts.union(mu,x,y), type="o", pch=20)

adf.test(x)   ### x is NOT Stationary, Unit Root Present

adf.test(y)   ### y is NOT stationary, Unit Root Present

po.test(cbind(x,y) )  ### x & y are cointegrated!

x.y.coint <- Arima(x, xreg=y)  ### Model x as a function of y
x.y.coint                      ### x = 0.67y  -- make sense?

plot(x.y.coint$residuals, type='o', pch=20)  ### appears stationary
adf.test(x.y.coint$residuals)                ### Dickey Fuller says it is Stationary!



