################################
## Daily Exchange rates for 
##    UK pounds
##    Euro
##    New Zealand Dollars
## 
## to US Dollars from January 2004 to December 2007
xrates <- read.csv("us_rates.csv")

xrates <- ts(xrates[,2:4], start=c(2004,1), frequency=365)
xrates.test <- window(xrates, start=c(2008,1) )
xrates <- window(xrates, end=c(2007,365))

plot(xrates, main="Exchange Rates to US Dollars")
### Each looks non-stationary (random-walk ish)

adf.test(xrates[,1])
adf.test(xrates[,2])
adf.test(xrates[,3])

### Dickey Fuller confirms

po.test(xrates[,1:2])    ## UK & NZ not cointegrated
po.test(xrates[,c(1,3)]) ## UK & EU maybe cointegrated?
po.test(xrates[,2:3])    ## NZ & EU not cointegrated

### Suppose we wished to model the UK exchange rate
uk <- xrates[,1]
eu <- xrates[,3]
plot(uk)   ## random walk right!
plot(diff(uk))
acf(diff(uk))
pacf(diff(uk))
### Looks like a Random Walk with maybe AR(1)?
Arima(uk, order=c(1,1,0), include.drift=TRUE)
Arima(uk, order=c(1,1,0) )
### Sure looks like an ARIMA(1,1,0) 
### 


# but... it is cointegrated with the EU...

uk.eu.coint <- Arima(uk, xreg=eu)
uk.eu.coint
plot(uk.eu.coint$residuals)  ### Maybe stationary?? Maybe not?
acf(uk.eu.coint$residuals)   ## This is consistent with a Very Strong AR process
pacf(uk.eu.coint$residuals)  ## So perhaps not-stationary, or maybe strong AR?

ar(uk.eu.coint$residuals)    ## AIC says a big AR(p) would work...

## What happens if we difference.
plot(diff(uk.eu.coint$residuals))
acf(diff(uk.eu.coint$residuals))
pacf(diff(uk.eu.coint$residuals))  

ar(diff(uk.eu.coint$residuals))  ## Another Big AR(p)

### Okay, let's compare some different fits

## Cointegration with AR(3)
Arima(uk, order=c(3,0,0), xreg=eu)
Arima(uk, order=c(1,1,0), xreg=eu)
Arima(uk, order=c(4,1,0), xreg=eu)

Arima(uk, order=c(2,0,0), xreg=eu)   ### Maybe the best???

Arima(uk, order=c(1,1,0) )           ### Straight up ARIMA model

## So the cointegrated Model with AR(2) has the best AIC, AICc and BIC.
## It may allow us to forecast the UK exchange rate too if we had EU measurements.


############################################
## Below I am trying some forecasting
## The cointegrated model is doing bad. 
## I think I am screwing something up, but out of time!

uk.coint <- Arima(uk, order=c(2,0,0), xreg=eu)
uk.arima <- Arima(uk, order=c(1,1,0))

uk.coint.forecast <- forecast(uk.coint, xreg=xrates.test[,3])
uk.arima.forecast <- forecast(uk.arima, h=15)

plot(uk.coint.forecast, include=5)
lines(xrates.test[,1])

plot(uk.arima.forecast, include=5)
lines(xrates.test[,1])
