#####
## This is how to read in a csv file

tmp <- read.csv("gdpUnemp.csv")

gdp <- ts(tmp[,2], start=c(1980,1), frequency=4)
unemp <- ts(tmp[,3], start=c(1980,1), frequency=4)
financial.series <- ts.union(gdp, unemp)
plot(financial.series, main="Quarterly GDP & Unemployment")
acf(financial.series)

### Use nonparametric smoothing to get a quick model
## for how the series connect.
gdp.model <- stl(financial.series[,1], s.window="periodic")
unemp.model <- stl(financial.series[,2], s.window="periodic")

plot(gdp.model)
plot(unemp.model)

### Get the "random" part, or the underlying error terms
gdp.random <- gdp.model$time.series[,3]
unemp.random <- unemp.model$time.series[,3]
acf(gdp.random)  ## Maybe an AR(1) or MA(1)?
pacf(unemp.random)  ## Closer to noise?
### Combine them back into one
random.series <- ts(cbind(gdp.random, unemp.random), start=c(1980,1), frequency=4)

plot(random.series)
acf(random.series)
ccf(gdp.random, unemp.random)

######## 
## we see cross-correlation at lag 0, this is instantaneous correlation.
##    this should make sense in the context of the data, both are financial
##    indicators.  Higher GDP, people tend to be working. Likewise, high
##    unemployment tends to be the economy is not growing
##
## But we also see something at lag -1.  This indicates that this quarters's GDP
##    is correlated with next quarters Unemployment. So we can use this feature
##    to forecast next month's unemployment!!
##
## and we see something at lag -2, this indicates this quarters GDP should
##    incluence two quarters from now Unemployment





plot(unemp)
unemp.arima <- Arima(unemp, order=c(3,1,0) )
forecast(unemp.arima, h=1)

gdp.arima <- Arima(log(gdp), order=c(1,1,0), include.drift=TRUE)


lm.unemp.gdp <- lm(diff(unemp[2:140]) ~ diff(log(gdp[1:139]) ) )
summary(lm.unemp.gdp)
plot(as.ts(lm.unemp.gdp$residuals))
plot(diff(as.ts(lm.unemp.gdp$residuals)))
acf((lm.unemp.gdp$residuals))
pacf((lm.unemp.gdp$residuals))
unemp.trim <- unemp[2:140]
Arima(unemp.trim, order=c(2,1,0), xreg=diff(log(gdp[1:140])))

