co2 <- read.csv("alertCO2.csv")
co2 <- ts(co2$CO2, start=c(1986,1), frequency=12)

plot(co2, main="Alert, Canada CO2", xlab="Time", ylab="CO2 (ppm)")

co2.train <- window(co2, end=c(2013,12))
co2.test <- window(co2, start=c(2014,1))

TIME <- 1:length(co2.train)
SIN <- COS <- matrix(nrow=length(TIME), ncol=6, 0)
for(i in 1:6) {
  SIN[,i] <- sin(2*pi*i*TIME/12)
  COS[,i] <- cos(2*pi*i*TIME/12)
}
SIN <- SIN[,-6]

summary(lm(co2.train ~ TIME + SIN + COS))
SIN <- SIN[,-c(4,5)]
COS <- COS[,-c(4,5,6)]
co2.lm <- lm(co2.train ~ TIME + SIN + COS)
summary(co2.lm)
plot(as.ts(co2.lm$residuals))
acf(co2.lm$residuals)
pacf(co2.lm$residuals)

################### 
## Clearly not stationary residuals
## What have we not modeled?
## The plot of residuals in time looks random-walk esc
## Should we difference???
plot(diff(as.ts(co2.lm$residuals)))
acf(diff(as.ts(co2.lm$residuals)))
pacf(diff(as.ts(co2.lm$residuals)))
##### Huge spike at lag 1 corresponding to a value of about -4? Maybe decaying?
ar(diff(co2.lm$residuals))  ### An AR(13)!!!!
ar13 <- Arima(diff(as.ts(co2.lm$residuals)), order=c(13,0,0), include.mean=TRUE)
ar13
acf(ar13$residuals)
pacf(ar13$residuals)  ### At least we got everything

### A routine that we can use sometimes to help us get started
auto.arima(diff(co2.lm$residuals))   ### Says an ARMA(4,4)

######
## So talking a regression term, 6 sin/cos terms and an ARMA(4,4) potentially
##
## We can fix some terms in the ARMA(4,4) really a ARMA(3,4)
Arima(diff(co2.lm$residuals), order=c(5,0,3), include.mean=FALSE)
arma53 <- Arima(diff(co2.lm$residuals), order=c(5,0,3), include.mean=FALSE, fixed=c(NA,0,NA,0,NA,0,0,NA))
arma53
acf(arma53$residuals)
pacf(arma53$residuals)

#### Close to everything?  Perhaps good enough. 

### Refit everything

XREG = cbind(TIME, SIN, COS)
lm.fit <- Arima(co2.train, order=c(5,1,3), xreg=XREG, fixed=c(NA,0,NA,0,NA,0,0,NA, NA,NA,NA,NA,NA,NA,NA))
lm.fit
acf(lm.fit$residuals)
pacf(lm.fit$residuals)




##############################
## Restart!
## Data is clearly highly seasonal
plot(diff(co2.train, lag=12))  ### Random walk esc
plot(diff(diff(co2.train,lag=12)))
diff.co2 <- diff(diff(co2.train,lag=12))
acf(diff.co2)
pacf(diff.co2)  ## Maybe seasonal MA?
Arima(co2.train, order=c(1,1,0), seasonal=list(order=c(1,1,0), period=12))
Arima(co2.train, order=c(0,1,1), seasonal=list(order=c(0,1,1), period=12))
### MA looks better
ma11 <- Arima(co2.train, order=c(0,1,1), seasonal=list(order=c(0,1,1), period=12))
ma11
acf(ma11$residuals)
pacf(ma11$residuals)   ### Maybe good enough?  I would argue as good as the regression


################### 
## Let's forecast!
TIME.NEW <- seq(length(TIME)+1, length(TIME)+12, by=1)
SIN.NEW <- COS.NEW <- matrix(nrow=length(TIME.NEW), ncol=3, 0)
for(i in 1:3) {
  SIN.NEW[,i] <- sin(2*pi*i*TIME.NEW/12)
  COS.NEW[,i] <- cos(2*pi*i*TIME.NEW/12)
}
XREG.NEW <- cbind(TIME.NEW, SIN.NEW, COS.NEW)
lm.forecast <- forecast(lm.fit, h=12, xreg=XREG.NEW)
plot(lm.forecast, include=12)
lines(co2.test, lwd=3, lty=3)

arima.forecast <- forecast(ma11, h=12)
plot(arima.forecast, include=12)
lines(co2.test, lwd=3, lty=3)

tab <- cbind(co2.test, lm.forecast$mean, arima.forecast$mean)
colnames(tab) <- c("CO2 Observed", "LM Forecast", "ARIMA Forecast")
tab

lm.mse <- mean( (co2.test - lm.forecast$mean)^2 )
arima.mse <- mean( (co2.test - arima.forecast$mean)^2 )

lm.mse
arima.mse

###############
## We see in this example the ARIMA is better
##
## But note the simplicity of the Seasonal ARIMA model.  We estimated 2 terms!!!
## compared to the linear model with 12