require(forecast)
require(WeightedPortTest)

data(AirPassengers)
AP <- AirPassengers
plot(AP, main="International Airline Passengers", ylab="Monthly Totals")
### Data appears to have an increasing variance, so take a log
log.AP <- log(AP)
plot(log.AP, main="International Airline Passengers", ylab="Log(Monthly Totals)")

### We see a trend and strong seasonal components. We could handle this 
### using a deterministic model (as we did before) or we can try differencing.
## Lag 12 differences to handle the seasonal aspect
diff.12.AP <- diff(log.AP, lag=12)
plot(diff.12.AP)
### The lag-12 differences resemble a random walk?? Difference again
diff.AP <- diff(diff.12.AP)
plot(diff.AP)

### This appears close to stationary, although there could be a change in variance.
### Is this noise though??? 
acf(diff.AP, lag.max=30)
pacf(diff.AP, lag.max=30)

### There are major correlation jumps at lags 1 and 12 (one year)
### There is some indication maybe an AR process could handle the low level 
### correlations as there appears to be some level of decay at lags 1 & 3
### but we also see something in the PACF at lag 3, so maybe an MA?
### There also could be something at lag 9-months. 
###
### Let's try some candidate models, note we can include the differencing
### in the arima() function as part of the integrated model
### Based on the ACF and PACF plots, it seems like a SMA(1) could handle
### the correlation at lag 12. 
### 
arima1 <- Arima(log.AP, order=c(0,1,1), seasonal=list(order=c(0,1,1), period=12) )
arima2 <- Arima(log.AP, order=c(1,1,0), seasonal=list(order=c(0,1,1), period=12) )
arima3 <- Arima(log.AP, order=c(0,1,1), seasonal=list(order=c(1,1,0), period=12) )
arima4 <- Arima(log.AP, order=c(1,1,0), seasonal=list(order=c(1,1,0), period=12) )

arima1
arima2
arima3
arima4

acf(arima1$residuals, lag.max=30)
pacf(arima1$residuals, lag.max=30)


### Let's refit the whole thing now
log.AP.fit <- Arima(log.AP, order=c(0,1,1), seasonal=list(order=c(0,1,1), period=12))
acf(log.AP.fit$residuals)
pacf(log.AP.fit$residuals)
Weighted.Box.test(log.AP.fit$residuals, lag=30, fitdf=2, type="Ljung")
Weighted.Box.test(log.AP.fit$residuals, lag=30, fitdf=2, type="Monti")

log.AP.pred <- forecast(log.AP.fit, h=48)
plot(log.AP.pred)

################
################
## Big idea... why this approach over the deterministic model
## 
## This modeling approach is simpler to estimate, no slopes, no harmonic stuff
## 
## Also, the deterministic approach makes teh major assumption that things will
## not change! So what if the increasing trend here were to level off?
## The deterministic approach will get really bad, however with a differencing
## type approach, if we observed more data that leveled off, our forecast for the 
## future would follow suit since a future prediction depends on the previous 
## term and observation of that season, rather than some deterministic
## equation. 
