set.seed(54321)
theta1 <- 0.5
theta4 <- -0.5
Mod(polyroot(c(1, theta1, 0, 0, theta4, theta1*theta4)))  ### So this is invertible!
tmp <- arima.sim(n=120, model=list(ma=c(theta1, 0, 0, theta4, theta1*theta4)))
x <- ts(tmp, start=c(1980,1), frequency=4)

plot(x)
acf(x, lwd=2)
pacf(x, lwd=2)

ar(x)    ### First a purely AR(p) model via AIC (agrees with ACF & PACF?)
Arima(x, order=c(6,0,0))   # Can simplify 
ar6 <- Arima(x, order=c(6,0,0), include.mean=FALSE)
ar6

#### What about an MA approach?
ma3 <- Arima(x, order=c(0,0,3), include.mean=TRUE)
ma3
acf(ma3$residuals, lwd=3)
pacf(ma3$residuals, lwd=2)  ### stuff at lag 7

ma4 <- Arima(x, order=c(0,0,4), include.mean=TRUE)
ma4
acf(ma4$residuals, lwd=3)
pacf(ma4$residuals, lwd=2)  ### Still Stuff at lag 7
### Really just fitting an MA(1) in these two

ma8 <- Arima(x, order=c(0,0,8), include.mean=FALSE)
ma8
acf(ma8$residuals, lwd=3)   ### Got almost everything?
pacf(ma8$residuals, lwd=2)  ### Yep
### We got everything, maybe simplify
ma8 <- Arima(x, order=c(0,0,8), include.mean=FALSE, fixed=c(NA,0,0,NA,NA,0,NA,NA))
acf(ma8$residuals, lwd=3)   ### Got almost everything?
pacf(ma8$residuals, lwd=2)  ### Yep
ma8


## Look at the original ACF & PACF again
acf(x, lwd=2)    # Decaying or cutting off?
pacf(x, lwd=2)   # Big jumps at lag 1 and 4 (the period!!!)
### Combine, the two tell me something seasonal!  Also there is something
### at lag 1, so maybe a seasonal AR or MA process with stuff at lag 1 & 4
Arima(x, order=c(1,0,0), seasonal=list(order=c(1,0,0), period=4), include.mean=FALSE)
Arima(x, order=c(0,0,1), seasonal=list(order=c(0,0,1), period=4), include.mean=FALSE)

#### Choose the MA based model
###
sma.model <- Arima(x, order=c(0,0,1), seasonal=list(order=c(0,0,1), period=4), include.mean=FALSE)
acf(sma.model$residuals, lwd=2)
pacf(sma.model$residuals, lwd=2)

###########
# We pretty much captured everything.
###########