### Lab #11 example
##
## first, open the file retail.dat
##
## This is a dataset from your textbook and is also in the TSA library

############
## An EDA/visual inspection of the series
retail <- ts(scan("retail.dat",skip=1), start=c(1986,1), fr=12)
plot(retail, main="U.K. Retail Sales", ylab="billion of pounds", type="o", pch=20)

##########
## Sure looks like we need a log transformation
log.retail <- log(retail)
plot(log.retail, main="U.K. Retail Sales", ylab="log billion pounds", type="o", pch=20)


#############################
# Most of the time in Time Series Analysis we are interested in predicting...
# so let's setup a training and testing set.
retail.train <- window(retail, end=c(2006,3))
retail.test <- window(retail, start=c(2006,4))

log.retail.train <- log(retail.train)
TIME <- 1:length(log.retail.train)
SIN <- COS <- matrix(nrow=length(log.retail.train), ncol=6, 0)
SEASONS <- cycle(log.retail.train)
for(i in 1:6) {
  COS[,i] <- cos(2*pi*i*TIME/12)
  SIN[,i] <- sin(2*pi*i*TIME/12)
}
SIN <- SIN[,-6]
lm.harmonic <- lm(log.retail.train ~ TIME + COS + SIN )
lm.seasons <- lm(log.retail.train ~ TIME + factor(SEASONS) )
summary(lm.harmonic)
summary(lm.seasons)
########################
## Pick your poison, in either case we need 10-terms to model the seasonality
## I choose the harmonic model just because we know each sin/cos term is indepent
## Let's peak at the residuals
plot(lm.harmonic$residuals, type='o', main="Residuals in Time Plot", pch=20)
#####
## Not-stationary??? Perhaps a quadratic trend?
##
## the ACF shows a slow decay, consistent with non-stationary
acf(lm.harmonic$residuals, lwd=2)
pacf(lm.harmonic$residuals)

### Perhaps we need to model a quadratic trend?
##
## Although not always the case, it is possible when fitting higher order polynomial
## equations in R, we could run into computational issues. Basically, our largest
## "TIME" value is 243, and when you square that you get 59049, the numerical algorithms
## that fit regression can start to run into numeric errors with large inputs
## So it is recommended to "standardize" the TIME coefficient.
STD.TIME <- (TIME - mean(TIME))/sd(TIME)
## Note this does not change much in our original model
summary(lm(log.retail.train ~ STD.TIME + COS + SIN))
## THe coefficient has changed but the t-value and p-value are the same on TIME

### Now let's add the quadradic and remove the unnecessary sign term
SIN <- SIN[,-4]
lm.quad.harmonic <- lm(log.retail.train ~ STD.TIME + I(STD.TIME^2) + COS + SIN )
summary(lm.quad.harmonic)

#####################
## Some residual analysis
plot(lm.quad.harmonic$residuals, type='o', main="Residuals in Time Plot", pch=20)
### Closer to stationary but perhaps a strong AR term

acf(lm.quad.harmonic$residuals, lwd=2)
pacf(lm.quad.harmonic$residuals, lwd=2)

### See the peak at lag 12  -- seasonal ARMA???
res.arima30 <- arima(lm.quad.harmonic$residuals, order=c(3,0,0), include.mean=FALSE)
res.arima30
acf(res.arima30$residual, lwd=2)
pacf(res.arima30$residual, lwd=2)

res.sarma <- arima(lm.quad.harmonic$residuals, order=c(3,0,0), seasonal=list(order=c(0,0,1), period=12), include.mean=FALSE)
res.sarma
acf(res.sarma$residual, lwd=2)
pacf(res.sarma$residual, lwd=2)  ## Got everything???

####################
## time to refit everything!!!
XREG <- cbind(STD.TIME, I(STD.TIME^2), COS, SIN)
colnames(XREG) <- c("std.time", "std.time2", "cos.1", "cos.2", "cos.3", "cos.4", "cos.5", "cos.6", "sin.1", "sin.2", "sin.3", "sin.5")
log.retail.lm <- Arima(log.retail.train, order=c(3,0,0), seasonal=list(order=c(0,0,1), period=12), xreg=XREG)
log.retail.lm

#### Check our residuals really quick...
plot(log.retail.lm$residuals, type='o', pch=20)  ## Noise?
acf(log.retail.lm$residuals, lwd=2)
pacf(log.retail.lm$residuals, lwd=2)

#################################3
## Looks like we got it!
##
## So we have a quadratic trend plus intercept, 10 harmonic terms and 4 ARMA terms
## 17 total terms plus the variance estimate!
## 
## A note, we lose some interpretability with the trend & quadratic coefficients
## since our time units have been changed. Initially one time unit was one month
## but now it is one standardized unit. We can transform back relatively easily
## by mulitplying the coefficient by the increments in STD.TIME
0.2676*(STD.TIME[2]-STD.TIME[1])
-0.0399*(STD.TIME[2]-STD.TIME[1])^2

####################################
## Seasonal ARIMA approach
plot(log.retail.train, type="o", pch=20)
### Clearly not stationary, trend with seasonality
plot(diff(log.retail.train), type="o", pch=20)
### Still seasonal
plot(diff(diff(log.retail.train), lag=12), type="o", pch=20)
### Now stationary
acf(diff(diff(log.retail.train)), lwd=2 )
pacf(diff(diff(log.retail.train)), lwd=2 )

tmp.arima <- Arima(log.retail.train, order=c(0,1,1), seasonal=list(order=c(0,1,0), period=12) )
acf(tmp.arima$residuals, lwd=2)
pacf(tmp.arima$residuals, lwd=2)
### Missing that seasonal term?
tmp.arima <- Arima(log.retail.train, order=c(0,1,1), seasonal=list(order=c(0,1,1), period=12) )
acf(tmp.arima$residuals, lwd=2)
pacf(tmp.arima$residuals, lwd=2)
## For comparison, let's try an seasonal AR modl
tmp.arima <- Arima(log.retail.train, order=c(1,1,0), seasonal=list(order=c(1,1,0), period=12) )
acf(tmp.arima$residuals, lwd=2)
pacf(tmp.arima$residuals, lwd=2)
## Not capturing the early stuff.  Maybe combine?
tmp.arima <- Arima(log.retail.train, order=c(0,1,1), seasonal=list(order=c(1,1,0), period=12) )
tmp.arima
acf(tmp.arima$residuals, lwd=2)
pacf(tmp.arima$residuals, lwd=2)
### the auto.arima() function says to do an SARIMA(2,1,0)x(0,1,2)
tmp.arima <- Arima(log.retail.train, order=c(2,1,0), seasonal=list(order=c(0,1,2), period=12) )
acf(tmp.arima$residuals, lwd=2)
pacf(tmp.arima$residuals, lwd=2)
### It gets it all, better than the seasonal MA I initially fit???
## Not entirely clear which to pick. AIC & BIC are better for the auto.arima approach.
## But most of the time how things foreacst is what is important
## Let's check out both and how they forecast

log.retail.sarima0101 <- Arima(log.retail.train, order=c(0,1,1), seasonal=list(order=c(0,1,1), period=12) )
log.retail.sarima2002 <- Arima(log.retail.train, order=c(2,1,0), seasonal=list(order=c(0,1,2), period=12) )

###
## Okay, let's forecast
##
## First we need to do some calculations for that big quadradic linear model
NEW.TIME <- seq(max(TIME)+1, max(TIME+12), 1)
NEW.SIN <- NEW.COS <- matrix(nrow=12, ncol=6, 0)
for(i in 1:6) {
  NEW.SIN[,i] <- sin(2*pi*i*NEW.TIME/12)
  NEW.COS[,i] <- cos(2*pi*i*NEW.TIME/12)
}
NEW.SIN <- NEW.SIN[,-c(4,6)]  # Remove 4th & 6th column
## Put the NEW.TIME on the same scale as the standardized Times
STD.NEW.TIME <- (NEW.TIME - mean(TIME))/sd(TIME)
tail(STD.TIME)
STD.NEW.TIME  ## Looks correct!
NEW.XREG <- cbind(STD.NEW.TIME, I(STD.NEW.TIME^2), NEW.COS, NEW.SIN)
quad.forecast <- forecast(log.retail.lm, h=12, xreg=NEW.XREG)

### Bit easier with the ARIMA model

sarima0101.forecast <- forecast(log.retail.sarima0101, h=12)
sarima2002.forecast <- forecast(log.retail.sarima2002, h=12)

plot(quad.forecast, include=18, main="Quadratic Linear Model + SARMA Forecast")
lines(log(retail.test), lwd=3, lty=2)

plot(sarima0101.forecast, include=18)
lines(log(retail.test), lwd=3, lty=2)

plot(sarima2002.forecast, include=18)
lines(log(retail.test), lwd=3, lty=2)

#### Not clear what is best?
predict.table <- cbind(log(retail.test), quad.forecast$mean,
                     sarima0101.forecast$mean, sarima2002.forecast$mean)
colnames(predict.table) <- c("Observed", "Quad Forecast", "SARIMA(0,1,1)(0,1,1)", "SARIMA(2,1,0)(0,1,2)")
predict.table

##################
## Still not entirely clear
mspe.table <- cbind(mean((quad.forecast$mean-log(retail.test))^2),
                    mean((sarima0101.forecast$mean-log(retail.test))^2),
                    mean((sarima2002.forecast$mean-log(retail.test))^2) )
rownames(mspe.table) <- "MSPE"
colnames(mspe.table) <- c("Quad Forecast", "SARIMA(0,1,1)(0,1,1)", "SARIMA(2,1,0)(0,1,2)")
mspe.table


#### So it seems the SARIMA(0,1,1)(0,1,1) is the best at forecasting
#### It is also the simplest model!

######################################
### But we built everything on a log scale, so let's get back to the original units
retail.forecast <- exp(sarima0101.forecast$mean)
plot(retail.forecast, lwd=2, col="blue", main="Forecasted with Observed",
     ylim=c(min(retail.forecast,retail.test), max(retail.forecast,retail.test)),
     ylab="Billions of Pounds")
lines(retail.test, lwd=3, lty=2)

#### We seem to consistently underestimate.  Recall Jensen's inequality & discussion...
### The predicted values are biased!  
### We should fix that bias by using the variance of each predictor
##
## Re-predict/forecast using the predict function.

sarima0101.predict <- predict(log.retail.sarima0101, n.ahead=12, se.fit=TRUE)
sarima0101.predict

####
## Remember how to make it unbiased
## exp(y.hat_h + 0.5*Var(error^2_h))
unbiased.predict <- exp(sarima0101.predict$pred + 0.5*sarima0101.predict$se^2)

retail.table <- cbind(retail.test, retail.forecast, unbiased.predict)
colnames(retail.table) <- c("Observed", "Exp(SARIMA forecast)", "Biased Corrected")
retail.table

mspe.table <- cbind(mean((retail.forecast-retail.test)^2),
                    mean((unbiased.predict-retail.test)^2) )
rownames(mspe.table) <- "MSPE"
colnames(mspe.table) <- c("Biased", "Unbiased")
mspe.table
### A modest improvement, but an improvement none-the-less!!

#####
## Visually it is hard to see a difference
unbiased.predict
retail.forecast <- exp(sarima0101.forecast$mean)
plot(retail.forecast, lwd=4, col="blue", main="Forecasted with Observed",
     ylim=c(min(retail.forecast,retail.test), max(retail.forecast,retail.test)),
     ylab="Billions of Pounds")
lines(unbiased.predict, lwd=2, lty=2, col="red")
lines(retail.test, lwd=3, lty=2)


# Fin
