

# Load any support libraries
library(shinydashboard)
#library(shiny)
#library(rCharts)
library(dplyr)
#library(shinyjs)
library(ggplot2)
library(RColorBrewer)
library(foreign)
library(DT)

outputDir <- "~/"

saveData <- function(data) {
  data <- t(data)
  # Create a unique file name
  fileName <- sprintf("%s_%s.csv", as.integer(Sys.time()), digest::digest(data))
  # Write the file to the local system
  write.csv(
    x = data,
    file = file.path(outputDir, fileName), 
    row.names = FALSE, quote = TRUE
  )
}

saveData2 <- function(data) {
  data <- as.data.frame(t(data))
  if (exists("responses")) {
    responses <<- rbind(responses, data)
  } else {
    responses <<- data
  }
}

loadData2 <- function() {
  if (exists("responses")) {
    responses
  }
}

source('LoadingCode.R', local=FALSE)

loadData <- function() {
  # Read all the files into a list
  files <- list.files( full.names = TRUE,pattern='.csv')
  data <- lapply(files, read.csv, stringsAsFactors = FALSE) 
  # Concatenate all data together into one data.frame
  data <- do.call(rbind, data)
  data
}

#source("C:/Users/Cody/Documents/GSS Research/DataCleaning.R")
fields <- c("var","foo","yearVar","iihs","nhtsa")
ui <- dashboardPage(
  
  dashboardHeader(title = "GSS Dashboard"),
  dashboardSidebar(
    tags$head(
      tags$style(HTML("
                      .sidebar { height: 90vh; overflow-y: auto; }
                      " )
      )
      ),
    sidebarMenu(
      menuItem("Overview", tabName = "dashboard", icon = icon("th")),
      menuItem("Make Overview", tabName = "makeyo", icon = icon("th")),
      menuItem("Manual Entry", icon = icon("th"), tabName = "widgets", badgeColor = "green"),
      menuItem("Accident Conditions Tool", icon = icon("th"), tabName = "Accidents")
    )),
  dashboardBody(
    
    tabItems(
          tabItem(tabName = "dashboard",
                  fluidPage(
                  box(width=10,
                  helpText("Numbers on top of bars represent # of cars in each rating"),
              plotOutput(outputId = "IIHSPlot"))),
              #plotOutput(outputId = "overviewheatIIHS"))),
              fluidPage(
              box(width=10,
                helpText("Numbers on top of bars represent # of cars in each rating"),
              plotOutput(outputId = "NHTSAPlot")))),
              #plotOutput(outputId = "overviewheatNHTSA")))),
              
          tabItem(tabName = "makeyo",
                  fluidRow(
                    box(
                      uiOutput("blockbymake")
                    )
                  ),
                  fluidRow(
                    box(
                      helpText("Numbers on top of bars represent # of cars in each rating"),
                      plotOutput(outputId = "blockplotNHTSA")
                    ),
                    box(
                      helpText("Numbers on top of bars represent # of cars in each rating"),
                      plotOutput(outputId= "blockplotIIHS")
                    )
                  )
          ),
          tabItem(tabName = "widgets",
                fluidRow(  
                      box(collapsible = TRUE,
                        uiOutput("MakeChoice"),
                        uiOutput("checkbGroups"),
                        uiOutput("yearly"),
                        actionButton("submit", "Save the Vehicle")
              )),
            
            fluidRow(
              box(collapsible = TRUE,width=10,
                  DT::dataTableOutput("responses")))) ,
           
            
           
      
    
        tabItem(tabName="Accidents",
                fluidRow(
                  box(
                    helpText("Must choose at least one of each to generate graphs"),
                    checkboxGroupInput("ChooseSev","Maximum Severity of Accident",choices=c("No Injury"=0,
                                                                                 "Possible Injury"=1,
                                                                                 "Minor Injury"=2,
                                                                                 "Serious Injury"=3,
                                                                                 "Injured, Severity Unknown"=4,
                                                                                 "Fatality"=5)),
                    br(), 
                    checkboxGroupInput("ChooseSpeed"," Was the vehicle Speeding?",choices=c("Yes" =TRUE,
                                                                                            "No" = FALSE)),
                    br(),
                    checkboxGroupInput("ChooseDamage","Type of Damage to Vehicle",choices=c("No Damage"=0,
                                                                                            "Minor Damage"=2,
                                                                                            "Moderate Damage"=4,
                                                                                            "Severe Damage"=6
                                                                                            )),
                    br(),
                    checkboxGroupInput("ChooseAlch"," Was there Alchohol Invloved?",choices=c("Yes"=TRUE,
                                                                                         "No"=FALSE))
                    
                    
                                                                                          
                  )
                ),
                fluidRow(
                  box(width=8,collapsible = TRUE,collapsed = TRUE,title="IIHS Accident Overview",
                      plotOutput(outputId = "accidentplotIIHS",width = "700px"),
                      br(),
                      plotOutput(outputId = "accidentheatIIHS",width="700px"))),
                fluidRow(
                  box(width=8,collapsible = TRUE,collapsed = TRUE,title="NHTSA Accident Overview",
                      plotOutput(outputId = "accidentplotNHTSA",width = "700px"),
                      br(),
                      plotOutput(outputId = "accidentheatNHTSA",width="700px")))
                )
    )
  )
)



server <- function(input, output,session) {
  values = reactiveValues()
  datasetInput = reactive({
    values$makes = input$var
    values$models = input$foo
    values$years = input$yearVar
    
    
    
  })
  
  
  
  
  
  
  
  
  formDat2 = reactive({
    fulldatGroup2 = filter(fulldatGroup,grepl(as.character(input$var), Make.x),
                           grepl(as.character(input$foo), Model.x),
                           Year.x == input$yearVar)
    data2 = c(as.character(fulldatGroup2[1,1]),as.character(fulldatGroup2[1,2]),
              as.numeric(fulldatGroup2[1,3]),as.numeric(fulldatGroup2[1,7]),
              as.numeric(fulldatGroup2[1,8]),as.numeric(fulldatGroup2[1,5]),as.numeric(fulldatGroup2[1,6]))
    data2
  })
  
  observeEvent(input$submit, {
    saveData2(formDat2())
    saveData(formDat2())
    #session$reload()
  })
  
  
  output$responses <- DT::renderDataTable({
    input$submit
    datadata = loadData2()
    datadata = datadata[!duplicated(datadata),]
    DT::datatable(datadata, options = list(pageLength = 25),colnames=c('ID'=1,"Make"=2,"Model"=3,"Year"=4,"IIHS"=5,"NHTSA"=6,"AvgInj"=7,"NumAcc"=8))
    
    
  })
  
  
  
  output$MakeChoice = renderUI({
    selectInput("var", 
                label = "Choose a Make",
                choices = as.list(sort(unique(as.character(fulldat2$Make.x)))))
  })
  
  output$checkbGroups = renderUI({
    newdat = filter(fulldat2,as.character(fulldat2$Make.x)==as.character(input$var))
    selectInput(input = 'foo',label='Choose a Model',
                choices= unique(sort(as.character(newdat$Model.x))))
  })
  
  
  output$yearly = renderUI({
    yeardat = filter(fulldat2,as.character(fulldat2$Make.x)==as.character(input$var),as.character(fulldat2$Model.x)==as.character(input$foo))
    selectInput(input='yearVar',label= 'Choose a Year',
                choices = unique(sort(as.numeric(as.character(yeardat$Year.x)))))
    
  })
  
  output$blockbymake = renderUI({
    selectInput(input='blockit',label="Choose a Make to Analyze",
                choices = as.list(sort(unique(as.character(fulldat2$Make.x)))))
  })
  
  # output$overviewheatIIHS = renderPlot({
  #   thisdat = fulldatGroup %>%
  #     group_by(Make.x,IIHS) %>%
  #     mutate(numAcc = length(Make.x))
  #   
  #   ggplot(thisdat, aes(x = IIHS, y = Make.x))+ geom_tile(aes(fill = numAcc)) + scale_fill_gradient(name = 'Number of Vehicles', low = 'white', high = 'red') + theme(axis.title.y = element_blank())+
  #     ggtitle("Heat Map of Number of Vehicles by IIHS Rating by Make")+
  #     theme(plot.title = element_text(size = 15))
  #   
  #   
  # })
  # output$overviewheatNHTSA = renderPlot({
  #   thisdat = fulldatGroup %>%
  #     group_by(Make.x,NHTSA) %>%
  #     mutate(numAcc = length(Make.x))
  #   
  #   ggplot(thisdat, aes(x = NHTSA, y = Make.x))+ geom_tile(aes(fill = numAcc)) + scale_fill_gradient(name = 'Number of Vehicles', low = 'white', high = 'red') + theme(axis.title.y = element_blank())+
  #     ggtitle("Heat Map of Number of Vehicles by NHTSA Rating by Make")+
  #     theme(plot.title = element_text(size = 15))
  #   
  #   
  # })
  
  
  output$accidentplotIIHS = renderPlot({
    thisdat = fulldat2 %>%
            filter(MAX_VSEV %in% input$ChooseSev,isSpeed %in% input$ChooseSpeed,DEFORMED %in% input$ChooseDamage,VEH_ALCH %in% input$ChooseAlch)%>%
            group_by(Make.x,Model.x,Year.x) %>%
            summarise(countInj = mean(NUM_INJV,na.rm=TRUE),
                numAcc = length(MAX_VSEV),
                IIHS=  getmode(IIHS),
                NHTSA=  getmode(NHTSA)) 
    
    thisdat2 = thisdat %>%
      group_by(IIHS) %>%
      summarise(countInj1 = weighted.mean(countInj,numAcc,na.rm=TRUE),
                numVar = length(countInj))
    
    g7 = ggplot(thisdat2,aes(x=factor(IIHS),y=countInj1,fill="green")) + geom_bar(stat="identity",colour="darkgreen") + 
      xlab("Crash-Safety Ratings") + ylab("Average Injuries per Rating") + theme(legend.position='none')+
      ggtitle("Average Injuries by IIHS Crash-Safety Rating") +
      theme(plot.title = element_text(color="#666666", face="bold", size=22, hjust=0.5))+
      geom_text(aes(label=numVar),vjust=-0.3,size=5,fontface = "bold")+theme_bw()+guides(fill=FALSE)
      
    
    print(g7)
    
  })
  
  
  output$accidentplotNHTSA = renderPlot({
    thisdat = fulldat2 %>%
      filter(MAX_VSEV %in% input$ChooseSev,isSpeed %in% input$ChooseSpeed,DEFORMED %in% input$ChooseDamage,VEH_ALCH %in% input$ChooseAlch)%>%
      group_by(Make.x,Model.x,Year.x) %>%
      summarise(countInj = mean(NUM_INJV,na.rm=TRUE),
                numAcc = length(MAX_VSEV),
                IIHS=  getmode(IIHS),
                NHTSA=  getmode(NHTSA)) 
    
    thisdat2 = thisdat %>%
      group_by(NHTSA) %>%
      summarise(countInj1 = weighted.mean(countInj,numAcc,na.rm=TRUE),
                numVar = length(countInj))
    
    g7 = ggplot(thisdat2,aes(x=factor(NHTSA),y=countInj1,fill="green")) + geom_bar(stat="identity",colour="darkgreen") + 
      xlab("Crash-Safety Ratings") + ylab("Average Injuries per Rating") + theme(legend.position='none')+
      ggtitle("Average Injuries by NHTSA Crash-Safety Rating") +
      theme(plot.title = element_text(color="#666666", face="bold", size=22, hjust=0.5))+
      geom_text(aes(label=numVar),vjust=-0.3,size=5,fontface = "bold")+theme_bw()+guides(fill=FALSE)
      
    
    print(g7)
    
  })
  
  
  # output$accidentheatIIHS = renderPlot({
  #   thisdat = fulldat2 %>%
  #     filter(MAX_VSEV %in% input$ChooseSev,isSpeed %in% input$ChooseSpeed,DEFORMED %in% input$ChooseDamage,VEH_ALCH %in% input$ChooseAlch)%>%
  #     group_by(Make.x,Model.x,Year.x) %>%
  #     summarise(numAcc = length(MAX_VSEV),
  #               IIHS=  getmode(IIHS),
  #               NHTSA=  getmode(NHTSA))
  #  
  #   ggplot(thisdat, aes(x = IIHS, y = Make.x))+ geom_tile(aes(fill = numAcc)) + scale_fill_gradient(name = 'Number of Vehicles', low = 'white', high = 'red') + theme(axis.title.y = element_blank())+
  #     ggtitle("Vehicle Heat Map of Number of Vehicles that Fit the Accident Types by IIHS Rating")+
  #     theme(plot.title = element_text(size = 15))
  #   
  #   
  # })
  # output$accidentheatNHTSA = renderPlot({
  #   if(length(input$ChooseSev)>0 & length(input$ChooseSpeed) > 0 & length(input$ChooseDamage) > 0 & length(input$ChooseAlch)>0){
  #   thisdat = fulldat2 %>%
  #     filter(MAX_VSEV %in% input$ChooseSev,isSpeed %in% input$ChooseSpeed,DEFORMED %in% input$ChooseDamage,VEH_ALCH %in% input$ChooseAlch)%>%
  #     group_by(Make.x,Model.x,Year.x) %>%
  #     summarise(numAcc = length(MAX_VSEV),
  #               IIHS=  getmode(IIHS),
  #               NHTSA=  getmode(NHTSA))
  #   
  #   ggplot(thisdat, aes(x = NHTSA, y = Make.x))+ geom_tile(aes(fill = numAcc)) + scale_fill_gradient(name = 'Number of Vehicles', low = 'white', high = 'red') + theme(axis.title.y = element_blank())+
  #     ggtitle("Vehicle Heat Map of Number of Vehicles that Fit the Accident Types by NHTSA Rating")+
  #     theme(plot.title = element_text(size = 15))
  #   
  #   }
  #   
  # })
  
  
  
  
  output$IIHSPlot = renderPlot({
    
    
    
    g3 = ggplot(injnum1,aes(x=factor(IIHS),y=countInj1,fill="green")) + geom_bar(stat="identity",colour="darkgreen") + 
      xlab("Crash-Safety Ratings") + ylab("Average Injuries per Rating") + theme(legend.position='none')+
      ggtitle("Average Injuries by IIHS Crash-Safety Rating") +
      theme(plot.title = element_text(color="#666666", face="bold", size=22, hjust=0.5))+
      geom_text(aes(label=numVar),vjust=-0.3,size=5,fontface = "bold")+theme_bw()+guides(fill=FALSE)
      
   
    print(g3)
    
  })
  
  output$NHTSAPlot = renderPlot({
    
    g4 = ggplot(injnum2,aes(x=factor(NHTSA),y=countInj1,fill="green")) + geom_bar(stat="identity",colour="darkgreen") + 
      xlab("Crash-Safety Ratings") + ylab("Average Injuries per Rating") + theme(legend.position='none')+
      ggtitle("Average Injuries by NHTSA Crash-Safety Rating") +
      theme(plot.title = element_text( color="#666666", face="bold", size=22, hjust=0.5))+
      geom_text(aes(label=numVar),vjust=-0.3,size=5,fontface = "bold")+theme_bw()+guides(fill=FALSE)
      
     
    print(g4)
  })
  
  output$blockplotNHTSA = renderPlot({
    blockdat = filter(fulldatGroup,Make.x==as.character(input$blockit))
    
    filtereddat2 = blockdat %>%
      group_by(NHTSA) %>%
      summarise(countInj1 = mean(countInj,na.rm=TRUE),
                majorSev1 = mean(majorSev, na.rm=TRUE),
                numAcc2 = length(countInj))
    
    g5 = ggplot(filtereddat2,aes(x=factor(NHTSA),y=countInj1,fill="green")) + geom_bar(stat="identity",colour="darkgreen") + 
      xlab("Crash-Safety Ratings") + ylab("Average Injuries per Rating") +
      ggtitle(paste("Average Injuries by NHTSA Crash-Safety Rating for",as.character(input$blockit))) + 
      geom_text(aes(label=numAcc2),vjust=-0.3,size=5,fontface = "bold")+
      theme(plot.title = element_text( color="#666666", face="bold", size=22, hjust=0.5))+theme_bw()+
      theme(legend.position='none')+guides(fill=FALSE)
    
        print(g5)
     
  })
  
  output$blockplotIIHS = renderPlot({
    blockdat = filter(fulldatGroup,Make.x==as.character(input$blockit))
    
    filtereddat2 = blockdat %>%
      group_by(IIHS) %>%
      summarise(countInj1 = mean(countInj,na.rm=TRUE),
                majorSev1 = mean(majorSev, na.rm=TRUE),
                numAcc2 = length(countInj))
    
    g5 = ggplot(filtereddat2,aes(x=factor(IIHS),y=countInj1,fill="green")) + geom_bar(stat="identity",colour="darkgreen") + 
      xlab("Crash-Safety Ratings") + ylab("Average Injuries per Rating") +
      ggtitle(paste("Average Injuries by IIHS Crash-Safety Rating for",as.character(input$blockit))) + 
      geom_text(aes(label=numAcc2),vjust=-0.3,size=5,fontface = "bold")+
      theme(plot.title = element_text( color="#666666", face="bold", size=22, hjust=0.5))+theme_bw()+
      theme(legend.position='none')+guides(fill=FALSE)
    
    print(g5)
    
  })
  
  
  # output$interPlotIIHS = renderPlot({
  #   
  # 
  #  
  #     input$submit
  #     isolate({
  #       
  #       
  #      datadata = loadData()
  #      datadata = datadata[!duplicated(datadata),]
  #      
  #      
  #      fulldatGroup2 = filter(fulldatGroup, Make.x %in% (as.character(datadata$V1)),
  #                             Model.x %in% (as.character(datadata$V2)),
  #                             Year.x %in% (as.numeric(as.character(datadata$V3))))
  #       filtereddat = fulldatGroup2 %>%
  #         group_by(IIHS) %>%
  #         summarise(countInj1 = mean(countInj,na.rm=TRUE),
  #                   majorSev1 = mean(majorSev, na.rm=TRUE),
  #                   numAcc2 = length(countInj))
  #       g4 = ggplot(filtereddat,aes(x=factor(IIHS),y=countInj1,fill="green")) + geom_bar(stat="identity",colour="darkgreen") + 
  #         xlab("Crash-Safety Ratings") + ylab("Average Injuries per Rating") +
  #         ggtitle("Average Injuries by IIHS Crash-Safety Rating") + 
  #         geom_text(aes(label=numAcc2),vjust=-0.3,size=5,fontface = "bold")+
  #         theme(plot.title = element_text( color="#666666", face="bold", size=22, hjust=0.5))+theme_bw()+
  #         theme(legend.position='none')+guides(fill=FALSE)
  #       if(!(sum(is.nan(filtereddat$countInj1))==nrow(filtereddat))){
  #         if(!(sum(is.na(filtereddat$IIHS))==nrow(filtereddat))){
  #       print(g4)
  #         }
  #       }
  #       
  #   
  #     
  #   })
  # })
  
  
  # output$interPlotNHTSA = renderPlot({
  #   
  #   
  #   
  #   input$submit
  #   isolate({
  #     
  #     
  #     datadata = loadData()
  #     datadata = datadata[!duplicated(datadata),]
  #     
  #     
  #     fulldatGroup2 = filter(fulldatGroup, Make.x %in% (as.character(datadata$V1)),
  #                            Model.x %in% (as.character(datadata$V2)),
  #                            Year.x %in% (as.numeric(as.character(datadata$V3))))
  #     filtereddat = fulldatGroup2 %>%
  #       group_by(NHTSA) %>%
  #       summarise(countInj1 = mean(countInj,na.rm=TRUE),
  #                 majorSev1 = mean(majorSev, na.rm=TRUE),
  #                 numAcc2 = length(countInj))
  #     g4 = ggplot(filtereddat,aes(x=factor(NHTSA),y=countInj1,fill="green")) + geom_bar(stat="identity",colour="darkgreen") + 
  #       xlab("Crash-Safety Ratings") + ylab("Average Injuries per Rating") +
  #       ggtitle("Average Injuries by NHTSA Crash-Safety Rating") + 
  #       geom_text(aes(label=numAcc2),vjust=-0.3,size=5,fontface = "bold")+
  #       theme(plot.title = element_text( color="#666666", face="bold", size=22, hjust=0.5))+theme_bw()+
  #       theme(legend.position='none')+guides(fill=FALSE)
  #     if(!(sum(is.nan(filtereddat$countInj1))==nrow(filtereddat))){
  #       if(!(sum(is.na(filtereddat$NHTSA))==nrow(filtereddat))){
  #         print(g4)
  #       }
  #     }
  #     
  #     
  #     
  #   })
  # })

}

shinyApp(ui, server)
